/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Common Exception Declarations
 */

#ifndef SCALESTACK_COMMON_EXCEPTION_H
#define SCALESTACK_COMMON_EXCEPTION_H

#include <exception>

#include <ScaleStack/Common/Macros.h>

namespace ScaleStack
{

namespace Common
{

/**
 * This class allows exceptions to be thrown with custom error mesages.
 */
class SCALESTACK_API Exception: public std::exception
{
public:

  /**
   * This constructor formats the context and message into a static buffer.
   *
   * @param[in] context Context where the exception occurred.
   * @param[in] message Message describing the exception.
   */
  Exception(const char* context, const char* message) throw();

  /**
   * Return the formatted buffer.
   */
  const char* what() const throw();

private:

  /* Use a static sized buffer so there is no risk of throwing exceptions. */
  char _message[SCALESTACK_MAX_MESSAGE_SIZE];
};

} /* namespace Common */

} /* namespace ScaleStack */

#endif /* SCALESTACK_COMMON_EXCEPTION_H */
