/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Notification Definitions
 */

#include "config.h"

#include <cerrno>

#include <unistd.h>

#include <ScaleStack/Common/Exception.h>
#include <ScaleStack/Common/Notification.h>

namespace ScaleStack
{

namespace Common
{

Notification::Notification(void)
{
  if (pipe(_pipe) == -1)
    throw Exception("Notification", "Could not create pipe");
}

Notification::~Notification()
{
  close(_pipe[0]);
  close(_pipe[1]);
}

void Notification::send(void)
{
  if (write(_pipe[1], "\0", 1) == -1 && errno != EINTR)
    throw Exception("Notification", "Could not write to pipe");
}

void Notification::wait(void)
{
  char buffer[1024];
  if (read(_pipe[0], buffer, sizeof(buffer)) == -1 && errno != EINTR)
    throw Exception("Notification", "Could not read from pipe");
}

} /* namespace Common */

} /* namespace ScaleStack */
