/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Notification Declarations
 */

#ifndef SCALESTACK_COMMON_NOTIFICATION_H
#define SCALESTACK_COMMON_NOTIFICATION_H

#include <ScaleStack/Common/Macros.h>

namespace ScaleStack
{

namespace Common
{

/**
 * This class provides a way for different threads to notify each other.
 */
class SCALESTACK_API Notification
{
public:

  Notification(void);

  ~Notification();

  /**
   * Send a notification.
   */
  void send(void);

  /**
   * Wait for a notification.
   */
  void wait(void);

private:

  /**
   * Don't allow copying of objects.
   */
  Notification(const Notification&);

  /**
   * Don't allow assignment of objects.
   */
  Notification& operator=(const Notification&);

  int _pipe[2];
};

} /* namespace Common */

} /* namespace ScaleStack */

#endif /* SCALESTACK_COMMON_NOTIFICATION_H */
