/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Common Test Declarations and Macro Definitions
 */

#ifndef SCALESTACK_COMMON_TEST_H
#define SCALESTACK_COMMON_TEST_H

#include "config.h"

#include <cassert>
#include <exception>
#include <iostream>
#include <string>

#include <ScaleStack/Kernel/Core.h>

using namespace std;
using namespace ScaleStack;

#define TEST_SUITE_BEGIN \
int main(int _test_argc, char* _test_argv[]) \
{ \
  size_t TEST_LOG_THRESHOLD = 0; \
  if (_test_argc == 2) \
    TEST_LOG_THRESHOLD = string(_test_argv[1]).size(); \
  if (TEST_LOG_THRESHOLD > 0) \
    cout << endl << __FILE__ << endl;

#define TEST_SUITE_END \
  return 0; \
}

#define TEST_BEGIN_NO_CORE(name) \
if (TEST_LOG_THRESHOLD > 0) \
  cout << "---------- TESTING " << #name << endl; \
{ \

#define TEST_BEGIN(name) \
  TEST_BEGIN_NO_CORE(name) \
  Kernel::Core core; \
  core.setLogThreshold(TEST_LOG_THRESHOLD); \
  core.addModuleOptionValue("path", "", "ScaleStack/.libs");


#define TEST_END \
} \
if (TEST_LOG_THRESHOLD > 0) \
  cout << "---------- PASSED" << endl << endl;

#define TEST_EXCEPTION_BEGIN \
{ \
  bool _test_exception = false; \
  try \
  {

#define TEST_EXCEPTION_END \
  } \
  catch (exception&) \
  { \
    _test_exception = true; \
  } \
  assert(_test_exception); \
}

#define TEST_EXCEPTION(code) \
do \
  TEST_EXCEPTION_BEGIN \
  code; \
  TEST_EXCEPTION_END \
while (false)

#define TEST_EVENT_SERVICE "EventLibevent"

#endif /* SCALESTACK_COMMON_TEST_H */
