/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Database Proxy Client Declarations
 */

#ifndef SCALESTACK_DATABASEPROXY_CLIENT_H
#define SCALESTACK_DATABASEPROXY_CLIENT_H

#include <ScaleStack/SocketTCP/Connection.h>

namespace ScaleStack
{

namespace DatabaseProxy
{

class ClientService;
class Connection;

class Client: public SocketTCP::Connection
{
public:

  Client(ClientService& client_service);

  /**
   * Set the connection that uses this client.
   */
  void setConnection(DatabaseProxy::Connection* connection,
                     bool need_handshake);

  /**
   * Notify client it should be yielded once current request is complete.
   */
  void yield(void);

  /**
   * Once connected, starting writing data from attached connection.
   */
  void connected(void);

  /**
   * See SocketTCP::Connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the connection from the read buffer.
   */
  void continueWrite(void);

  /**
   * See SocketTCP::Connection::flushWrite().
   */
  void flushWrite(void);

  /**
   * On error, notify attached connection to shutdown.
   */
  void error(void);

  /**
   * Process requests from attached connection on this clients thread.
   */
  void run(void);

private:

  /**
   * Don't allow copying of objects.
   */
  Client(const Client&);

  /**
   * Don't allow assignment of objects.
   */
  Client& operator=(const Client&);

  /**
   * Write data to the connection. This is used by both the read() and
   * continueWrite() methods.
   */
  size_t _connectionWrite(void);

  enum
  {
    SERVER_HANDSHAKE_PACKET,
    SERVER_HANDSHAKE_FLUSH,
    RESULT_PACKET,
    RESULT_FLUSH,
    COLUMN_PACKET,
    COLUMN_FLUSH,
    ROW_PACKET,
    ROW_FLUSH
  } _state, _next_state;
  bool _need_yield;
  bool _shutdown;
  size_t _size;
  size_t _packet_size;
  uint8_t* _buffer;
  DatabaseProxy::Connection* _connection;
};

} /* namespace DatabaseProxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_DATABASEPROXY_CLIENT_H */
