/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Database Proxy Client Service Declarations
 */

#ifndef SCALESTACK_DATABASEPROXY_CLIENTSERVICE_H
#define SCALESTACK_DATABASEPROXY_CLIENTSERVICE_H

#include <vector>

#include <ScaleStack/SocketTCP/ConnectionService.h>

namespace ScaleStack
{

namespace DatabaseProxy
{

class Client;
class Connection;

class ClientService: public SocketTCP::ConnectionService
{
public:

  ClientService(Kernel::Module& module);

  ~ClientService();

  /**
   * See SocketTCP::ConnectionService::addConnection().
   */
  SocketTCP::Connection* addConnection(void);

  /**
   * See SocketTCP::ConnectionService::removeConnection().
   */
  void removeConnection(SocketTCP::Connection* connection);

  /**
   * Get a client connection to proxy to.
   */
  Client* getClient(Connection* connection, bool need_handshake);

  /**
   * Yield a client to other pending transactions.
   */
  void yieldClient(Client* client);

  /**
   * Erase a connection from the queue.
   */
  void removeFromQueue(Connection* connection);

private:

  /**
   * Don't allow copying of objects.
   */
  ClientService(const ClientService&);

  /**
   * Don't allow assignment of objects.
   */
  ClientService& operator=(const ClientService&);

  size_t _max_clients;
  size_t _current_clients;
  pthread_mutex_t _queue_mutex;
  std::vector<Connection*> _waiting_connections;
  std::vector<Client*> _free_clients;
};

} /* namespace DatabaseProxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_DATABASEPROXY_CLIENTSERVICE_H */
