/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Database Proxy Connection Declarations
 */

#ifndef SCALESTACK_DATABASEPROXY_CONNECTION_H
#define SCALESTACK_DATABASEPROXY_CONNECTION_H

#include <ScaleStack/SocketTCP/Connection.h>

namespace ScaleStack
{

namespace DatabaseProxy
{

class Client;
class Service;

class Connection: public SocketTCP::Connection
{
public:

  Connection(Service& service);

  /**
   * See SocketTCP::Connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the client from the read buffer.
   */
  void continueWrite(void);

  /**
   * See SocketTCP::Connection::flushWrite().
   */
  void flushWrite(void);

  /**
   * Start shutdown process.
   */
  void error(void);

  /**
   * Process shutdown requests, detaching from client.
   */
  void run(void);

  /**
   * Notify connection it can yielf the client now.
   */
  void yield(void);

private:

  /**
   * Don't allow copying of objects.
   */
  Connection(const Connection&);

  /**
   * Don't allow assignment of objects.
   */
  Connection& operator=(const Connection&);

  /**
   * Write data to the client. This is used by both the read() and
   * continueWrite() methods.
   */
  size_t _clientWrite(void);

  enum
  {
    CLIENT_HANDSHAKE_PACKET,
    CLIENT_HANDSHAKE_FLUSH,
    COMMAND_PACKET,
    COMMAND_FLUSH
  } _state;
  bool _need_handshake;
  bool _need_yield;
  bool _in_transaction;
  bool _shutdown;
  size_t _size;
  size_t _packet_size;
  uint8_t* _buffer;
  Client* _client;
};

} /* namespace DatabaseProxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_DATABASEPROXY_CONNECTION_H */
