/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Database Proxy Module Definitions
 */

#include "config.h"

#include <ScaleStack/DatabaseProxy/Service.h>
#include <ScaleStack/Kernel/Core.h>
#include <ScaleStack/Kernel/Module.h>

using namespace std;
using namespace ScaleStack;

static void options(Kernel::Module& module)
{
  SocketTCP::ListenerOptions(module, "7");
  module.addOption("client_host", "Host to proxy to.", "HOST", "127.0.0.1");
  module.addOption("client_port", "Port to proxy to.", "PORT", "12345");
  module.addOption("max_clients", "Maxiumum number of clients to use.",
                   "MAX", "1");
}

static void start(Kernel::Module& module)
{
  DatabaseProxy::Service* service = new DatabaseProxy::Service(module);
  try
  {
    module.getCore().addService("DatabaseProxy", service);
  }
  catch (exception&)
  {
    delete service;
    throw;
  }
}

static void stop(Kernel::Module& module)
{
  delete module.getCore().removeService("DatabaseProxy");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
