/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Database Proxy Service Definitions
 */

#include "config.h"

#include <ScaleStack/DatabaseProxy/Connection.h>
#include <ScaleStack/DatabaseProxy/Service.h>

namespace ScaleStack
{

namespace DatabaseProxy
{

/*
 * Public functions.
 */

Service::Service(Kernel::Module& module):
  SocketTCP::ConnectionService(module),
  _listener_service(module, *this),
  _client_service(module)
{
}

Service::~Service()
{
  _listener_service.shutdown();
  shutdown();
  _client_service.shutdown();
}

SocketTCP::Connection* Service::addConnection(void)
{
  return new Connection(*this);
}

void Service::removeConnection(SocketTCP::Connection* connection)
{
  delete connection;
}

Client* Service::getClient(Connection* connection, bool need_handshake)
{
  return _client_service.getClient(connection, need_handshake);
}

void Service::yieldClient(Client* client)
{
  _client_service.yieldClient(client);
}

void Service::removeFromQueue(Connection* connection)
{
  _client_service.removeFromQueue(connection);
}

} /* namespace DatabaseProxy */

} /* namespace ScaleStack */
