/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Database Proxy Service Declarations
 */

#ifndef SCALESTACK_DATABASEPROXY_SERVICE_H
#define SCALESTACK_DATABASEPROXY_SERVICE_H

#include <ScaleStack/DatabaseProxy/ClientService.h>
#include <ScaleStack/SocketTCP/ConnectionService.h>
#include <ScaleStack/SocketTCP/ListenerService.h>

namespace ScaleStack
{

namespace DatabaseProxy
{

class Client;
class Connection;

class Service: public SocketTCP::ConnectionService
{
public:

  Service(Kernel::Module& module);

  ~Service();

  /**
   * See SocketTCP::ConnectionService::addConnection().
   */
  SocketTCP::Connection* addConnection(void);

  /**
   * See SocketTCP::ConnectionService::removeConnection().
   */
  void removeConnection(SocketTCP::Connection* connection);

  /**
   * Get a client connection to proxy to.
   */
  Client* getClient(Connection* connection, bool need_handshake);

  /**
   * Yield a client to other pending transactions.
   */
  void yieldClient(Client* client);

  /**
   * Erase a connection from the queue.
   */
  void removeFromQueue(Connection* connection);

private:

  /**
   * Don't allow copying of objects.
   */
  Service(const Service&);

  /**
   * Don't allow assignment of objects.
   */
  Service& operator=(const Service&);

  SocketTCP::ListenerService _listener_service;
  ClientService _client_service;
};

} /* namespace DatabaseProxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_DATABASEPROXY_SERVICE_H */
