/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Echo Server Connection Definitions
 */

#include "config.h"

#include <ScaleStack/EchoServer/Connection.h>
#include <ScaleStack/EchoServer/Service.h>

namespace ScaleStack
{

namespace EchoServer
{

/*
 * Public methods.
 */

Connection::Connection(Service& service):
  SocketTCP::Connection(service),
  _size(),
  _buffer()
{
}

size_t Connection::read(uint8_t* buffer, size_t size)
{
  _size = size;
  _buffer = buffer;
  return _echo();
}

void Connection::flushWrite(void)
{
  consume(_echo());
}

/*
 * Private methods.
 */

size_t Connection::_echo(void)
{
  size_t written = write(_buffer, _size, true);
  _size -= written;
  _buffer += written;
  return written;
}

} /* namespace EchoServer */

} /* namespace ScaleStack */
