/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Echo Server Connection Declarations
 */

#ifndef SCALESTACK_ECHOSERVER_CONNECTION_H
#define SCALESTACK_ECHOSERVER_CONNECTION_H

#include <ScaleStack/SocketTCP/Connection.h>

namespace ScaleStack
{

namespace EchoServer
{

class Service;

class Connection: public SocketTCP::Connection
{
public:

  Connection(Service& service);

  /**
   * See SocketTCP::Connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * See SocketTCP::Connection::flushWrite().
   */
  void flushWrite(void);

private:

  /**
   * Don't allow copying of objects.
   */
  Connection(const Connection&);

  /**
   * Don't allow assignment of objects.
   */
  Connection& operator=(const Connection&);

  /**
   * Echo the data back to the connection by writing any data that was
   * read. This is used by both the read() and flushWrite() methods.
   */
  size_t _echo(void);

  size_t _size;
  uint8_t* _buffer;
};

} /* namespace EchoServer */

} /* namespace ScaleStack */

#endif /* SCALESTACK_ECHOSERVER_CONNECTION_H */
