/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Event Service Provider Declarations
 */

#ifndef SCALESTACK_EVENT_SERVICEPROVIDER_H
#define SCALESTACK_EVENT_SERVICEPROVIDER_H

#include <ScaleStack/Common/Macros.h>
#include <ScaleStack/Event/Handler.h>
#include <ScaleStack/Event/Service.h>
#include <ScaleStack/Kernel/Service.h>

namespace ScaleStack
{

namespace Event
{

class Handler;

/**
 * This is a base class that event service provider modules must implement.
 */
class SCALESTACK_API ServiceProvider: public Kernel::Service
{
public:

  ServiceProvider(Kernel::Module& module);

  virtual ~ServiceProvider();

  /**
   * See Service::addHandler().
   */
  virtual void addHandler(Handler* handler) = 0;

  /**
   * See Service::removeHandler().
   */
  virtual void removeHandler(Handler* handler);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  ServiceProvider(const ServiceProvider&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  ServiceProvider& operator=(const ServiceProvider&);
};

/*
 * Public methods.
 */

inline ServiceProvider::ServiceProvider(Kernel::Module& module):
  Kernel::Service(module)
{
}

inline ServiceProvider::~ServiceProvider()
{
}

inline void ServiceProvider::removeHandler(Handler* handler)
{
  handler->getService()._removeHandler(handler);
}

} /* namespace Event */

} /* namespace ScaleStack */

#endif /* SCALESTACK_EVENT_SERVICEPROVIDER_H */
