/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Event Test Module Definitions
 */

#include "config.h"

#include <cassert>

#include <ScaleStack/Event/Handler.h>
#include <ScaleStack/Event/Service.h>
#include <ScaleStack/Event/ServiceProvider.h>
#include <ScaleStack/Kernel/Core.h>
#include <ScaleStack/Kernel/Module.h>

using namespace std;

namespace ScaleStack
{

namespace Event
{

namespace Test
{

class ServiceProvider: public Event::ServiceProvider
{
public:
  ServiceProvider(Kernel::Module& module);
  void addHandler(Event::Handler* handler);
};

class Service: public Event::Service
{
public:
  Service(Kernel::Module& module);
  ~Service();
  void removeHandler(Event::Handler* handler);
};

class HandlerProvider: public Event::HandlerProvider
{
public:
  void remove(void);
  void setTimer(time_t timer);
  void setFileDescriptor(int file_descriptor);
  void watchRead(void);
  void watchWrite(void);
  void runNow(void);
};

/*
 * Public Service Provider methods.
 */

ServiceProvider::ServiceProvider(Kernel::Module& module):
  Event::ServiceProvider(module)
{
}

void ServiceProvider::addHandler(Event::Handler* handler)
{
  (void) handler;
}

/*
 * Public Service methods.
 */

Service::Service(Kernel::Module& module):
  Event::Service(module)
{
}

Service::~Service()
{
  shutdown();
}

void Service::removeHandler(Event::Handler* handler)
{
  delete handler;
}

/*
 * Public Handler Provider methods.
 */

void HandlerProvider::remove(void)
{
}

void HandlerProvider::setTimer(time_t)
{
}

void HandlerProvider::setFileDescriptor(int)
{
}

void HandlerProvider::watchRead(void)
{
}

void HandlerProvider::watchWrite(void)
{
}

void HandlerProvider::runNow(void)
{
}

} /* namespace Test */

} /* namespace Event */

} /* namespace ScaleStack */

using namespace ScaleStack;

static void start(Kernel::Module& module)
{
  Event::Test::ServiceProvider* service_provider = new Event::Test::ServiceProvider(module);
  module.getCore().addService("Event", service_provider);

  Event::Test::Service* service = new Event::Test::Service(module);
  module.getCore().addService("Event::Test", service);
}

static void stop(Kernel::Module& module)
{
  delete module.getCore().removeService("Event::Test");
  delete module.getCore().removeService("Event");
}

static void run(Kernel::Module& module)
{
  module.getCore().shutdown();
}

SCALESTACK_KERNEL_MODULE(NULL, start, stop, run, NULL);
