/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief libevent Event Module Definitions
 */

#include "config.h"

#include <exception>

#include <ScaleStack/EventLibevent/ServiceProvider.h>
#include <ScaleStack/Kernel/Core.h>
#include <ScaleStack/Kernel/Module.h>

using namespace std;
using namespace ScaleStack;

static void options(Kernel::Module& module)
{
  module.addOption("threads", "Number of event threads to use.", "THREADS", "1");
  module.addOption("thread_stack_size", "Size of the stack for threads.", "STACKSIZE", "0");
}

static void start(Kernel::Module& module)
{
  EventLibevent::ServiceProvider* service_provider = new EventLibevent::ServiceProvider(module);
  try
  {
    module.getCore().addService("Event", service_provider);
  }
  catch (exception&)
  {
    delete service_provider;
    throw;
  }
}

static void stop(Kernel::Module& module)
{
  delete module.getCore().removeService("Event");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
