/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief libevent Event Service Provider Declarations
 */

#ifndef SCALESTACK_EVENTLIBEVENT_SERVICEPROVIDER_H
#define SCALESTACK_EVENTLIBEVENT_SERVICEPROVIDER_H

#include <vector>

#include <ScaleStack/Common/Macros.h>
#include <ScaleStack/Event/ServiceProvider.h>

namespace ScaleStack
{

namespace EventLibevent
{

class HandlerProvider;
class Thread;

class SCALESTACK_API ServiceProvider: public Event::ServiceProvider
{
public:

  /**
   * Add the number of threads specified in the "threads" option.
   *
   * @param[in] module Module that the service is associated with.
   */
  ServiceProvider(Kernel::Module& module);

  ~ServiceProvider();

  /**
   * See Event::Service::addHandler().
   */
  void addHandler(Event::Handler* handler);

  /**
   * Remove handler provider that was previously added in addHandler().
   *
   * @param[in] handler_provider Handler provider to remove.
   */
  void removeHandlerProvider(HandlerProvider* handler_provider);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  ServiceProvider(const ServiceProvider&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  ServiceProvider& operator=(const ServiceProvider&);

  /**
   * Remove all threads that were created in the constructor.
   */
  void _removeThreads(void);

  std::vector<Thread*> _threads;
  std::vector<Thread*>::iterator _current_thread;
};

} /* namespace EventLibevent */

} /* namespace ScaleStack */

#endif /* SCALESTACK_EVENTLIBEVENT_SERVICEPROVIDER_H */
