/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Log Console Definitions
 */

#include "config.h"

#include <cerrno>
#include <cstdio>
#include <stdint.h>
#include <unistd.h>

#include <ScaleStack/Kernel/LogConsole.h>

using namespace std;

namespace ScaleStack
{

namespace Kernel
{

/*
 * Public methods.
 */

LogConsole::LogConsole()
{
}

void LogConsole::log(Log::Level level, const char* context, const char* message)
{
  /* Another buffer is used here so we can have one atomic write to stdout. */
  char buffer[SCALESTACK_MAX_MESSAGE_SIZE];
  uint32_t size = snprintf(buffer, SCALESTACK_MAX_MESSAGE_SIZE, "%6s [%s] %s\n",
                           getLevelString(level), context, message);
  if (size > SCALESTACK_MAX_MESSAGE_SIZE)
    size = SCALESTACK_MAX_MESSAGE_SIZE;
  if (write(1, buffer, size) == -1)
    fprintf(stderr, "Could not write to console: %d\n", errno);
}

} /* namespace Kernel */

} /* namespace ScaleStack */
