/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Log Console Declarations
 */

#ifndef SCALESTACK_KERNEL_LOGCONSOLE_H
#define SCALESTACK_KERNEL_LOGCONSOLE_H

#include <ScaleStack/Common/Macros.h>
#include <ScaleStack/Kernel/Log.h>

namespace ScaleStack
{

namespace Kernel
{

/**
 * This is a kernel log implementation that writes to standard output.
 */
class SCALESTACK_API LogConsole: public Log
{
public:

  LogConsole(void);

  /**
   * Write a log entry to standard output.
   *
   * @param[in] level Log level for this message.
   * @param[in] context Context string for this message.
   * @param[in] message Message to log.
   */
  void log(Level level, const char* context, const char* message);

private:

  /**
   * Don't allow copying of objects.
   */
  LogConsole(const LogConsole&);

  /**
   * Don't allow assignment of objects.
   */
  LogConsole& operator=(const LogConsole&);
};

} /* namespace Kernel */

} /* namespace ScaleStack */

#endif /* SCALESTACK_KERNEL_LOGCONSOLE_H */
