/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Option Tests
 */

#include <ScaleStack/Common/Test.h>

#include <ScaleStack/Kernel/Module.h>
#include <ScaleStack/Kernel/Option.h>

TEST_SUITE_BEGIN

TEST_BEGIN(constructorTrue)
  Kernel::Module& module = core.getOrAddModule("Module");
  Kernel::Option& option = module.addOption("Option", "Help", "Bool", "True");
  assert(&option.getModule() == &module);
  assert(option.getName() == "Option");
  assert(option.getHelp() == "Help");
  assert(option.getValueName() == "Bool");
  assert(option.getDefaultValue() == "True");
  assert(option.getValue() == "True");
  assert(option.getBoolValue() == true);
  assert(option.isValid() == true);
TEST_END

TEST_BEGIN(constructorFalse)
  Kernel::Module& module = core.getOrAddModule("Module");
  Kernel::Option& option = module.addOption("Option", "Help", "Bool", "False");
  assert(&option.getModule() == &module);
  assert(option.getName() == "Option");
  assert(option.getHelp() == "Help");
  assert(option.getValueName() == "Bool");
  assert(option.getDefaultValue() == "False");
  assert(option.getValue() == "False");
  assert(option.getBoolValue() == false);
  assert(option.isValid() == true);
TEST_END

TEST_BEGIN(constructorValue)
  Kernel::Module& module = core.getOrAddModule("Module");
  Kernel::Option& option = module.addOption("Option", "Help", "Value Name", "Default Value");
  assert(&option.getModule() == &module);
  assert(option.getName() == "Option");
  assert(option.getHelp() == "Help");
  assert(option.getValueName() == "Value Name");
  assert(option.getDefaultValue() == "Default Value");
  assert(option.getValue() == "Default Value");
  assert(option.isValid() == true);
TEST_END

TEST_BEGIN(loadOption)
  Kernel::Module& module = core.getOrAddModule("Kernel::TestOption");
  core.addModuleOptionValue("Kernel::TestOption", "Option4", "");
  core.addModuleOptionValue("Kernel::TestOption", "Option5", "");
  core.addModuleOptionValue("Kernel::TestOption", "Option6", "Value6");
  assert(module.getOption("Option4").isValid() == false);
  assert(module.getOption("Option5").isValid() == false);
  assert(module.getOption("Option6").isValid() == false);
  core.run();
  assert(module.getOption("Option1").isValid() == true);
  assert(module.getOption("Option2").isValid() == true);
  assert(module.getOption("Option3").isValid() == true);
  assert(module.getOption("Option4").isValid() == true);
  assert(module.getOption("Option5").isValid() == true);
  assert(module.getOption("Option6").isValid() == true);
TEST_END

TEST_BEGIN(badOption)
  core.getOrAddModule("TestOptionBad");
  TEST_EXCEPTION(core.run());
TEST_END

TEST_SUITE_END
