/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Service Declarations
 */

#ifndef SCALESTACK_KERNEL_SERVICE_H
#define SCALESTACK_KERNEL_SERVICE_H

#include <ScaleStack/Common/Macros.h>

namespace ScaleStack
{

namespace Kernel
{

class Module;

/**
 * This is the base class that all services should inherit from. It is a
 * common type to provide some level of safety when adding services to the
 * kernel core.
 */
class SCALESTACK_API Service
{
public:

  Service(Module& module);

  virtual ~Service();

  /**
   * Get the module associated with this service.
   */
  Module& getModule(void);

protected:

  Module& _module;

private:

  /**
   * Don't allow copying of objects.
   */
  Service(const Service&);

  /**
   * Don't allow assignment of objects.
   */
  Service& operator=(const Service&);
};

/*
 * Public methods.
 */

inline Service::Service(Module& module):
  _module(module)
{
}

inline Service::~Service()
{
}

inline Module& Service::getModule(void)
{
  return _module;
}

} /* namespace Kernel */

} /* namespace ScaleStack */

#endif /* SCALESTACK_KERNEL_SERVICE_H */
