/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief TestModuleChild Module Definitions
 */

#include "config.h"

#include <cassert>

#include <ScaleStack/Kernel/Core.h>
#include <ScaleStack/Kernel/Module.h>

using namespace ScaleStack;

static void start(Kernel::Module& module)
{
  assert(module.getName() == "Kernel::TestModuleChild");
  assert(module.getPathname() != "");
  assert(module.getVersion() == "TestModuleChild Version");
  assert(module.getAuthor() == "TestModuleChild Author");
  assert(module.getTitle() == "TestModuleChild Title");
  assert(module.getLicense() == "TestModuleChildLicense");
  assert(module.getDependencies() == "Kernel::TestModuleParent");
  assert(module.getParentCount() == 1);
  assert(module.isLoaded() == true);

  Kernel::Module& parent = module.getCore().getModule("Kernel::TestModuleParent");
  assert(parent.getTitle() == "TestModuleParent Title");
  assert(parent.isLoaded() == true);

  Kernel::Module& grand_parent = module.getCore().getModule("Kernel::TestModuleGrandParent");
  assert(grand_parent.getTitle() == "TestModuleGrandParent Title");
  assert(grand_parent.isLoaded() == true);
}

SCALESTACK_KERNEL_MODULE(NULL, start, NULL, NULL, NULL);
