/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief TestOption Module Definitions
 */

#include "config.h"

#include <cassert>

#include <ScaleStack/Kernel/Core.h>
#include <ScaleStack/Kernel/Module.h>
#include <ScaleStack/Kernel/Option.h>

using namespace ScaleStack;

static void options(Kernel::Module& module)
{
  module.addOption("Option1", "Help1", "Bool1", "True");
  module.addOption("Option2", "Help2", "Bool2", "False");
  module.addOption("Option3", "Help3", "ValueName3", "DefaultValue3");
  module.addOption("Option4", "Help4", "Bool4", "True");
  module.addOption("Option5", "Help5", "Bool5", "False");
  module.addOption("Option6", "Help6", "ValueName6", "DefaultValue6");
  module.addOption("Option7", "Help7", "Size7", "7");
}

static void start(Kernel::Module& module)
{
  assert(module.getOption("Option1").getBoolValue() == true);
  assert(module.getOption("Option2").getBoolValue() == false);
  assert(module.getOption("Option3").getValue() == "DefaultValue3");
  assert(module.getOption("Option4").getBoolValue() == true);
  assert(module.getOption("Option5").getBoolValue() == true);
  assert(module.getOption("Option6").getValue() == "Value6");
  assert(module.getOption("Option7").getSizeValue() == 7);
}

static void run(Kernel::Module& module)
{
  module.getCore().shutdown();
}

SCALESTACK_KERNEL_MODULE(options, start, NULL, run, NULL);
