/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Client Definitions
 */

#include "config.h"

#include <ScaleStack/Kernel/Module.h>
#include <ScaleStack/Proxy/Client.h>
#include <ScaleStack/Proxy/ClientService.h>
#include <ScaleStack/Proxy/Connection.h>

namespace ScaleStack
{

namespace Proxy
{

/*
 * Public methods.
 */

Client::Client(ClientService& client_service):
  SocketTCP::Connection(client_service),
  _shutdown(false),
  _size(),
  _buffer(),
  _connection()
{
}

void Client::setConnection(Proxy::Connection* connection)
{
  if (connection == NULL)
  {
    _shutdown = true;
    runNow();
    return;
  }

  _connection = connection;
}

void Client::connected(void)
{
  _connection->continueWrite();
}

size_t Client::read(uint8_t* buffer, size_t size)
{
  _size = size;
  _buffer = buffer;
  return _connectionWrite();
}

void Client::continueWrite(void)
{
  consume(_connectionWrite());
}

void Client::flushWrite(void)
{
  _connection->continueWrite();
}

void Client::error(void)
{
  _connection->error();
}

void Client::run(void)
{
  SocketTCP::Connection::run();

  if (_shutdown)
  {
    _connection->runNow();
    remove();
  }
}

/*
 * Private methods.
 */

size_t Client::_connectionWrite(void)
{
  size_t written = _connection->write(_buffer, _size, true);
  _size -= written;
  _buffer += written;
  return written;
}

} /* namespace Proxy */

} /* namespace ScaleStack */
