/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Client Declarations
 */

#ifndef SCALESTACK_PROXY_CLIENT_H
#define SCALESTACK_PROXY_CLIENT_H

#include <ScaleStack/SocketTCP/Connection.h>

namespace ScaleStack
{

namespace Proxy
{

class ClientService;
class Connection;

class Client: public SocketTCP::Connection
{
public:

  Client(ClientService& client_service);

  /**
   * Set the connection that uses this client.
   */
  void setConnection(Proxy::Connection* connection);

  /**
   * Once connected, starting writing data from attached connection.
   */
  void connected(void);

  /**
   * See SocketTCP::Connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the connection from the read buffer.
   */
  void continueWrite(void);

  /**
   * See SocketTCP::Connection::flushWrite().
   */
  void flushWrite(void);

  /**
   * On error, notify attached connection to shutdown.
   */
  void error(void);

  /**
   * Process requests from attached connection on this clients thread.
   */
  void run(void);

private:

  /**
   * Don't allow copying of objects.
   */
  Client(const Client&);

  /**
   * Don't allow assignment of objects.
   */
  Client& operator=(const Client&);

  /**
   * Write data to the connection. This is used by both the read() and
   * continueWrite() methods.
   */
  size_t _connectionWrite(void);

  bool _shutdown;
  size_t _size;
  uint8_t* _buffer;
  Proxy::Connection* _connection;
};

} /* namespace Proxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_PROXY_CLIENT_H */
