/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Client Service Definitions
 */

#include "config.h"

#include <ScaleStack/Kernel/Module.h>
#include <ScaleStack/Kernel/Option.h>
#include <ScaleStack/Proxy/Client.h>
#include <ScaleStack/Proxy/ClientService.h>

using namespace std;

namespace ScaleStack
{

namespace Proxy
{

/*
 * Public functions.
 */

ClientService::ClientService(Kernel::Module& module):
  SocketTCP::ConnectionService(module)
{
}

ClientService::~ClientService()
{
  shutdown();
}

SocketTCP::Connection* ClientService::addConnection(void)
{
  return new Client(*this);
}

void ClientService::removeConnection(SocketTCP::Connection* connection)
{
  delete connection;
}

Client* ClientService::getClient(Connection* connection)
{
  const string& host = _module.getOption("client_host").getValue();
  const string& port = _module.getOption("client_port").getValue();
  Client* client = static_cast<Client*>(addConnect(host, port));
  client->setConnection(connection);
  return client;
}

} /* namespace Proxy */

} /* namespace ScaleStack */
