/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Client Service Declarations
 */

#ifndef SCALESTACK_PROXY_CLIENTSERVICE_H
#define SCALESTACK_PROXY_CLIENTSERVICE_H

#include <ScaleStack/SocketTCP/ConnectionService.h>

namespace ScaleStack
{

namespace Proxy
{

class Client;
class Connection;

class ClientService: public SocketTCP::ConnectionService
{
public:

  ClientService(Kernel::Module& module);

  ~ClientService();

  /**
   * See SocketTCP::ConnectionService::addConnection().
   */
  SocketTCP::Connection* addConnection(void);

  /**
   * See SocketTCP::ConnectionService::removeConnection().
   */
  void removeConnection(SocketTCP::Connection* connection);

  /**
   * Get a client connection to proxy to.
   */
  Client* getClient(Connection* connection);

private:

  /**
   * Don't allow copying of objects.
   */
  ClientService(const ClientService&);

  /**
   * Don't allow assignment of objects.
   */
  ClientService& operator=(const ClientService&);
};

} /* namespace Proxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_PROXY_CLIENTSERVICE_H */
