/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Connection Definitions
 */

#include "config.h"

#include <ScaleStack/Kernel/Module.h>
#include <ScaleStack/Proxy/Client.h>
#include <ScaleStack/Proxy/Connection.h>
#include <ScaleStack/Proxy/Service.h>

namespace ScaleStack
{

namespace Proxy
{

/*
 * Public methods.
 */

Connection::Connection(Service& service):
  SocketTCP::Connection(service),
  _shutdown(false),
  _shutdown_write(false),
  _size(),
  _buffer(),
  _client(service.getClient(this))
{
}

size_t Connection::read(uint8_t* buffer, size_t size)
{
  _size = size;
  _buffer = buffer;
  return _clientWrite();
}

void Connection::continueWrite(void)
{
  consume(_clientWrite());
}

void Connection::flushWrite(void)
{
  if (!_shutdown)
    _client->continueWrite();
}

void Connection::readEOF(void)
{
  if (_shutdown)
    error();
  else
  {
    if (_size == 0)
      _client->shutdownWrite();
    else
      _shutdown_write = true;
  }
}

void Connection::error(void)
{
  _shutdown = true;
  runNow();
}

void Connection::run(void)
{
  if (_client == NULL)
  {
    remove();
    return;
  }

  if (_shutdown)
  {
    _client->setConnection(NULL);
    _client = NULL;
    return;
  }
}

/*
 * Private methods.
 */

size_t Connection::_clientWrite(void)
{
  if (_shutdown)
    return 0;

  size_t written = _client->write(_buffer, _size, true);
  _size -= written;
  _buffer += written;

  if (_shutdown_write && _size == 0)
    _client->shutdownWrite();

  return written;
}

} /* namespace Proxy */

} /* namespace ScaleStack */
