/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Connection Declarations
 */

#ifndef SCALESTACK_PROXY_CONNECTION_H
#define SCALESTACK_PROXY_CONNECTION_H

#include <ScaleStack/SocketTCP/Connection.h>

namespace ScaleStack
{

namespace Proxy
{

class Client;
class Service;

class Connection: public SocketTCP::Connection
{
public:

  Connection(Service& service);

  /**
   * See SocketTCP::Connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the client from the read buffer.
   */
  void continueWrite(void);

  /**
   * See SocketTCP::Connection::flushWrite().
   */
  void flushWrite(void);

  /**
   * Notify the client to shutdown the write socket if needed.
   */
  void readEOF(void);

  /**
   * Start shutdown process.
   */
  void error(void);

  /**
   * Process shutdown requests, detaching from client.
   */
  void run(void);

private:

  /**
   * Don't allow copying of objects.
   */
  Connection(const Connection&);

  /**
   * Don't allow assignment of objects.
   */
  Connection& operator=(const Connection&);

  /**
   * Write data to the client. This is used by both the read() and
   * continueWrite() methods.
   */
  size_t _clientWrite(void);

  bool _shutdown;
  bool _shutdown_write;
  size_t _size;
  uint8_t* _buffer;
  Client* _client;
};

} /* namespace Proxy */

} /* namespace ScaleStack */

#endif /* SCALESTACK_PROXY_CONNECTION_H */
