/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief Proxy Service Definitions
 */

#include "config.h"

#include <ScaleStack/Proxy/Connection.h>
#include <ScaleStack/Proxy/Service.h>

namespace ScaleStack
{

namespace Proxy
{

/*
 * Public functions.
 */

Service::Service(Kernel::Module& module):
  SocketTCP::ConnectionService(module),
  _listener_service(module, *this),
  _client_service(module)
{
}

Service::~Service()
{
  _listener_service.shutdown();
  shutdown();
  _client_service.shutdown();
}

SocketTCP::Connection* Service::addConnection(void)
{
  return new Connection(*this);
}

void Service::removeConnection(SocketTCP::Connection* connection)
{
  delete connection;
}

Client* Service::getClient(Connection* connection)
{
  return _client_service.getClient(connection);
}

} /* namespace Proxy */

} /* namespace ScaleStack */
