/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief TCP Socket Connection Service Definitions
 */

#include "config.h"

#include <exception>

#include <ScaleStack/SocketTCP/Connection.h>
#include <ScaleStack/SocketTCP/ConnectionService.h>

using namespace std;

namespace ScaleStack
{

namespace SocketTCP
{

/*
 * Public methods.
 */

ConnectionService::ConnectionService(Kernel::Module& module):
  Event::Service(module)
{
}

ConnectionService::~ConnectionService()
{
}

Connection* ConnectionService::addAccepted(int file_descriptor)
{
  Connection* connection = _addConnection();
  connection->setFileDescriptor(file_descriptor);
  return connection;
}

Connection* ConnectionService::addConnect(const std::string& host,
                                   const std::string& port)
{
  Connection* connection = _addConnection();
  connection->connect(host, port);
  return connection;
}

/*
 * Private methods.
 */

Connection* ConnectionService::_addConnection(void)
{
  Connection* connection = NULL;
  try
  {
    connection = addConnection();
    addHandler(connection);
  }
  catch (exception& e)
  {
    if (connection != NULL)
      removeConnection(connection);

    throw;
  }

  return connection;
}

void ConnectionService::removeHandler(Event::Handler* handler)
{
  removeConnection(static_cast<Connection*>(handler));
}

} /* namespace SocketTCP */

} /* namespace ScaleStack */
