/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief TCP Socket Listener Declarations
 */

#ifndef SCALESTACK_SOCKETTCP_LISTENER_H
#define SCALESTACK_SOCKETTCP_LISTENER_H

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>

#include <ScaleStack/Common/Macros.h>
#include <ScaleStack/Event/Handler.h>

namespace ScaleStack
{

namespace Kernel
{

class Module;

} /* namespace Kernel */

namespace SocketTCP
{

class ConnectionService;
class ListenerService;

class Listener: public Event::Handler
{
public:

  /**
   * Constructor that starts listening on the address described in addrinfo.
   *
   * @param[in] service Service this Listener is associated with.
   * @param[in] connection_service Service this Listener should add
   *  connections to when accepted.
   * @param[in] addrinfo Address information to use for socket.
   */
  Listener(ListenerService& service,
           ConnectionService& connection_service,
           struct addrinfo* addrinfo);

  ~Listener();

  /**
   * Being ready for read means there is a socket to accept. This accepts
   * a socket and creates a new connection for the service.
   */
  void readReady(int file_descriptor);

private:

  /**
   * Don't allow copying of objects.
   */
  Listener(const Listener&);

  /**
   * Don't allow assignment of objects.
   */
  Listener& operator=(const Listener&);

  int _file_descriptor;
  Kernel::Module& _module;
  ConnectionService& _connection_service;
};

} /* namespace SocketTCP */

} /* namespace ScaleStack */

#endif /* SCALESTACK_SOCKETTCP_LISTENER_H */
