/*
 * Scale Stack
 *
 * Copyright (C) 2010 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * Use and distribution licensed under the BSD license. See the
 * COPYING file in the root project directory for full text.
 */

/**
 * @file
 * @brief TCP Socket Listener Service Declarations
 */

#ifndef SCALESTACK_SOCKETTCP_LISTENERSERVICE_H
#define SCALESTACK_SOCKETTCP_LISTENERSERVICE_H

#include <string>

#include <ScaleStack/Event/Service.h>

namespace ScaleStack
{

namespace SocketTCP
{

class ConnectionService;
class Listener;

/**
 * Add common options for TCP listen sockets.
 *
 * @param[in] module Module that the service is associated with.
 * @param[in] port Default port to use.
 */
SCALESTACK_API
void ListenerOptions(Kernel::Module& module, const char* port);

class SCALESTACK_API ListenerService: public Event::Service
{
public:

  ListenerService(Kernel::Module& module,
                  ConnectionService& connection_service);

  ~ListenerService();

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  ListenerService(const ListenerService&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  ListenerService& operator=(const ListenerService&);

  /**
   * Add listeners for the host/port pair.
   */
  SCALESTACK_LOCAL
  void _addListeners(ConnectionService& connection_service,
                     std::string host, std::string port);

  /**
   * See Event::Service::removeHandler().
   */
  SCALESTACK_LOCAL
  void removeHandler(Event::Handler* handler);
};

} /* namespace SocketTCP */

} /* namespace ScaleStack */

#endif /* SCALESTACK_SOCKETTCP_LISTENERSERVICE_H */
